@@ -10,6 +10,7 @@ import copy
 
 import numpy as np
 from astropy import units
+from astropy.io import fits
 from astropy.coordinates import Angle
 import pytest
 
@@ -18,6 +19,9 @@ import pyuvdata.tests as uvtest
 import pyuvdata.utils as uvutils
 from pyuvdata.data import DATA_PATH
 
+from pyuvdata.uvbeam.tests.test_mwa_beam import filename as mwa_beam_file
+from pyuvdata.uvbeam.tests.test_cst_beam import cst_files, cst_yaml_file
+
 
 try:
     from astropy_healpix import HEALPix
@@ -26,6 +30,8 @@ try:
 except (ImportError):
     healpix_installed = False
 
+casa_beamfits = os.path.join(DATA_PATH, "HERABEAM.FITS")
+
 
 @pytest.fixture(scope="function")
 def uvbeam_data():
@@ -248,66 +254,54 @@ def test_properties(uvbeam_data):
             raise
 
 
-def test_deprecation_warnings_set_cs_params(cst_efield_2freq):
+def test_set_cs_params(cst_efield_2freq):
     """
-    Test the deprecation warnings in set_cs_params.
+    Test _set_cs_params.
     """
     efield_beam = cst_efield_2freq
     efield_beam2 = efield_beam.copy()
 
-    with uvtest.check_warnings(
-        DeprecationWarning, match="`set_cs_params` is deprecated"
-    ):
-        efield_beam2.set_cs_params()
-
+    efield_beam2._set_cs_params()
     assert efield_beam2 == efield_beam
 
 
-def test_deprecation_warnings_set_efield(cst_efield_2freq):
+def test_set_efield(cst_efield_2freq):
     """
-    Test the deprecation warnings in set_efield.
+    Test _set_efield parameter settings.
     """
     efield_beam = cst_efield_2freq
     efield_beam2 = efield_beam.copy()
 
-    with uvtest.check_warnings(DeprecationWarning, match="`set_efield` is deprecated"):
-        efield_beam2.set_efield()
+    efield_beam2._set_efield()
 
     assert efield_beam2 == efield_beam
 
 
-def test_deprecation_warnings_set_power(cst_power_2freq):
+def test_set_power(cst_power_2freq):
     """
-    Test the deprecation warnings in set_power.
+    Test _set_power parameter settings.
     """
     power_beam = cst_power_2freq
     power_beam2 = power_beam.copy()
 
-    with uvtest.check_warnings(DeprecationWarning, match="`set_power` is deprecated"):
-        power_beam2.set_power()
+    power_beam2._set_power()
 
     assert power_beam2 == power_beam
 
 
-def test_deprecation_warnings_set_antenna_type(cst_efield_2freq):
+def test_set_antenna_type(cst_efield_2freq):
     """
-    Test the deprecation warnings in set_simple and set_phased_array.
+    Test set_simple and set_phased_array parameter settings.
     """
     efield_beam = cst_efield_2freq
     efield_beam2 = efield_beam.copy()
 
-    with uvtest.check_warnings(DeprecationWarning, match="`set_simple` is deprecated"):
-        efield_beam2.set_simple()
+    efield_beam2._set_simple()
 
     assert efield_beam2 == efield_beam
 
     efield_beam._set_phased_array()
-    with uvtest.check_warnings(
-        DeprecationWarning, match="`set_phased_array` is deprecated"
-    ):
-        efield_beam2.set_phased_array()
-
-    assert efield_beam2 == efield_beam
+    assert efield_beam2 != efield_beam
 
 
 def test_errors():
@@ -527,7 +521,7 @@ def test_efield_to_power(cst_efield_2freq_cut, cst_power_2freq_cut, tmp_path):
     efield_beam.Naxes_vec = 3
     with pytest.raises(
         ValueError,
-        match="Conversion to power with 3-vector efields " "is not currently supported",
+        match="Conversion to power with 3-vector efields is not currently supported",
     ):
         efield_beam.efield_to_power()
 
@@ -1418,7 +1412,7 @@ def test_select_frequencies(cst_power_1freq, tmp_path):
         assert f in freqs_to_keep
 
     assert uvutils._check_histories(
-        old_history + "  Downselected to " "specific frequencies using pyuvdata.",
+        old_history + "  Downselected to specific frequencies using pyuvdata.",
         power_beam2.history,
     )
 
@@ -1458,7 +1452,7 @@ def test_select_frequencies(cst_power_1freq, tmp_path):
         assert f in power_beam.freq_array[0, chans_to_keep]
 
     assert uvutils._check_histories(
-        old_history + "  Downselected to " "specific frequencies using pyuvdata.",
+        old_history + "  Downselected to specific frequencies using pyuvdata.",
         power_beam2.history,
     )
 
@@ -1508,15 +1502,20 @@ def test_select_feeds(cst_efield_1freq):
         assert f in feeds_to_keep
 
     assert uvutils._check_histories(
-        old_history + "  Downselected to " "specific feeds using pyuvdata.",
+        old_history + "  Downselected to specific feeds using pyuvdata.",
         efield_beam2.history,
     )
 
+    # check with physical orientation strings:
+    efield_beam3 = efield_beam.select(feeds=["e"], inplace=False)
+
+    assert efield_beam2 == efield_beam3
+
     # check for errors associated with feeds not included in data
     with pytest.raises(
-        ValueError, match="Feed {f} is not present in the feed_array".format(f="N")
+        ValueError, match="Feed {f} is not present in the feed_array".format(f="p")
     ):
-        efield_beam.select(feeds=["N"])
+        efield_beam.select(feeds=["p"])
 
     # check for error with selecting polarizations on efield beams
     with pytest.raises(
@@ -1542,7 +1541,10 @@ def test_select_feeds(cst_efield_1freq):
         efield_beam.check()
 
 
-def test_select_polarizations(cst_power_1freq):
+@pytest.mark.parametrize(
+    "pols_to_keep", ([-5, -6], ["xx", "yy"], ["nn", "ee"], [[-5, -6]])
+)
+def test_select_polarizations(pols_to_keep, cst_power_1freq):
     power_beam = cst_power_1freq
 
     # generate more polarizations for testing by copying and adding several times
@@ -1568,21 +1570,44 @@ def test_select_polarizations(cst_power_1freq):
     )
 
     old_history = power_beam.history
-    pols_to_keep = [-5, -6]
 
     power_beam2 = power_beam.select(polarizations=pols_to_keep, inplace=False)
 
+    if isinstance(pols_to_keep[0], list):
+        pols_to_keep = pols_to_keep[0]
+
     assert len(pols_to_keep) == power_beam2.Npols
     for p in pols_to_keep:
-        assert p in power_beam2.polarization_array
+        if isinstance(p, int):
+            assert p in power_beam2.polarization_array
+        else:
+            assert (
+                uvutils.polstr2num(p, x_orientation=power_beam2.x_orientation)
+                in power_beam2.polarization_array
+            )
     for p in np.unique(power_beam2.polarization_array):
-        assert p in pols_to_keep
+        if isinstance(pols_to_keep[0], int):
+            assert p in pols_to_keep
+        else:
+            assert p in uvutils.polstr2num(
+                pols_to_keep, x_orientation=power_beam2.x_orientation
+            )
 
     assert uvutils._check_histories(
-        old_history + "  Downselected to " "specific polarizations using pyuvdata.",
+        old_history + "  Downselected to specific polarizations using pyuvdata.",
         power_beam2.history,
     )
 
+
+def test_select_polarizations_errors(cst_power_1freq):
+    power_beam = cst_power_1freq
+
+    # generate more polarizations for testing by copying and adding several times
+    while power_beam.Npols < 4:
+        new_beam = power_beam.copy()
+        new_beam.polarization_array = power_beam.polarization_array - power_beam.Npols
+        power_beam += new_beam
+
     # check for errors associated with polarizations not included in data
     with pytest.raises(
         ValueError,
@@ -2006,7 +2031,7 @@ def test_add(cst_power_1freq, cst_efield_1freq):
         UserWarning,
         "Only one of the UVBeam objects being combined has optional parameter",
     ):
-        beam1.__iadd__(beam2)
+        beam1 += beam2
 
     assert beam1.receiver_temperature_array is None
 
@@ -2018,16 +2043,23 @@ def test_add(cst_power_1freq, cst_efield_1freq):
         polarizations=power_beam.polarization_array[2:4], inplace=False
     )
     beam2.history += " testing the history. Read/written with pyuvdata"
-    beam1 += beam2
+    new_beam = beam1 + beam2
     assert uvutils._check_histories(
-        power_beam.history + "  Downselected to specific polarizations "
-        "using pyuvdata. Combined data along "
-        "polarization axis using pyuvdata. "
-        "testing the history.",
-        beam1.history,
+        power_beam.history + "  Downselected to specific polarizations using pyuvdata. "
+        "Combined data along polarization axis using pyuvdata. Unique part of next "
+        "object history follows.  testing the history.",
+        new_beam.history,
+    )
+    new_beam.history = power_beam.history
+    assert new_beam == power_beam
+
+    new_beam = beam1.__add__(beam2, verbose_history=True)
+    assert uvutils._check_histories(
+        power_beam.history + "  Downselected to specific polarizations using pyuvdata. "
+        "Combined data along polarization axis using pyuvdata. Next object history "
+        "follows. " + beam2.history,
+        new_beam.history,
     )
-    beam1.history = power_beam.history
-    assert beam1 == power_beam
 
     # ------------------------
     # Test failure modes of add function
@@ -2119,7 +2151,7 @@ def test_select_healpix_pixels(
         assert pi in pixels_to_keep
 
     assert uvutils._check_histories(
-        old_history + "  Downselected to " "specific healpix pixels using pyuvdata.",
+        old_history + "  Downselected to specific healpix pixels using pyuvdata.",
         beam_healpix2.history,
     )
 
@@ -2494,3 +2526,158 @@ def test_get_beam_functions(cst_power_1freq_cut_healpix):
         ValueError, match="Do not have the right polarization information"
     ):
         healpix_power_beam._get_beam(4)
+
+
+def test_generic_read_cst():
+    uvb = UVBeam()
+    uvb.read(
+        cst_files,
+        beam_type="power",
+        frequency=np.array([150e6, 123e6]),
+        feed_pol="y",
+        telescope_name="TEST",
+        feed_name="bob",
+        feed_version="0.1",
+        model_name="E-field pattern - Rigging height 4.9m",
+        model_version="1.0",
+        run_check=False,
+    )
+    assert uvb.check()
+
+
+@pytest.mark.parametrize(
+    "filename", [cst_yaml_file, mwa_beam_file, casa_beamfits],
+)
+def test_generic_read(filename):
+    """Test generic read can infer the file types correctly."""
+    uvb = UVBeam()
+    # going to check in a second anyway, no need to double check.
+    uvb.read(filename, run_check=False)
+    # hera casa beam is missing some parameters but we just want to check
+    # that reading is going okay
+    if filename == casa_beamfits:
+        # fill in missing parameters
+        uvb.data_normalization = "peak"
+        uvb.feed_name = "casa_ideal"
+        uvb.feed_version = "v0"
+        uvb.model_name = "casa_airy"
+        uvb.model_version = "v0"
+
+        # this file is actually in an orthoslant projection RA/DEC at zenith at a
+        # particular time.
+        # For now pretend it's in a zenith orthoslant projection
+        uvb.pixel_coordinate_system = "orthoslant_zenith"
+    assert uvb.check()
+
+
+def test_generic_read_bad_filetype():
+    uvb = UVBeam()
+    with pytest.raises(ValueError, match="File type could not be determined"):
+        uvb.read("foo")
+
+
+def test_generic_read_multi(tmp_path):
+    uvb = UVBeam()
+    uvb.read(mwa_beam_file, pixels_per_deg=1, freq_range=[100e6, 200e6])
+
+    uvb1 = uvb.select(frequencies=uvb.freq_array[0, ::2], inplace=False)
+    uvb2 = uvb.select(frequencies=uvb.freq_array[0, 1::2], inplace=False)
+    fname1 = str(tmp_path / "test_beam1.beamfits")
+    fname2 = str(tmp_path / "test_beam2.beamfits")
+    uvb1.write_beamfits(fname1)
+    uvb2.write_beamfits(fname2)
+
+    uvb3 = UVBeam()
+    uvb3.read([fname1, fname2])
+    # the histories will be different
+    uvb3.history = uvb.history
+
+    assert uvb3 == uvb
+
+
+@pytest.mark.parametrize("skip", [True, False])
+@pytest.mark.parametrize("flip_order", [True, False])
+def test_generic_read_multi_bad_files(tmp_path, skip, flip_order):
+    uvb = UVBeam()
+    uvb = UVBeam()
+    uvb.read(mwa_beam_file, pixels_per_deg=1, freq_range=[100e6, 200e6])
+
+    uvb1 = uvb.select(frequencies=uvb.freq_array[0, ::2], inplace=False)
+    uvb2 = uvb.select(frequencies=uvb.freq_array[0, 1::2], inplace=False)
+    fname1 = str(tmp_path / "test_beam1.beamfits")
+    fname2 = str(tmp_path / "test_beam2.beamfits")
+    uvb1.write_beamfits(fname1)
+    uvb2.write_beamfits(fname2)
+
+    # Give file a bad beam type
+    fits.setval(fname1, "BTYPE", value="foobar")
+    uvb3 = UVBeam()
+    filenames = [fname1, fname2]
+    if flip_order:
+        # reverse the order to trigger other try/catch block
+        filenames = list(reversed(filenames))
+    if skip:
+        with uvtest.check_warnings(
+            UserWarning, f"Failed to read {filenames[0]} due to ValueError"
+        ):
+            uvb3.read(filenames, skip_bad_files=skip)
+        uvb3.history == uvb2.history
+        assert uvb3 == uvb2
+
+    else:
+
+        with pytest.raises(ValueError, match="Unknown beam_type: foobar, beam_type"):
+            uvb3.read(filenames, skip_bad_files=skip)
+
+
+def test_generic_read_all_bad_files(tmp_path):
+    uvb = UVBeam()
+    uvb = UVBeam()
+    uvb.read(mwa_beam_file, pixels_per_deg=1, freq_range=[100e6, 200e6])
+
+    uvb1 = uvb.select(frequencies=uvb.freq_array[0, ::2], inplace=False)
+    uvb2 = uvb.select(frequencies=uvb.freq_array[0, 1::2], inplace=False)
+    fname1 = str(tmp_path / "test_beam1.beamfits")
+    fname2 = str(tmp_path / "test_beam2.beamfits")
+    uvb1.write_beamfits(fname1)
+    uvb2.write_beamfits(fname2)
+    # Give files a bad beam type
+    fits.setval(fname1, "BTYPE", value="foobar")
+    fits.setval(fname2, "BTYPE", value="foobar")
+    uvb3 = UVBeam()
+    filenames = [fname1, fname2]
+    with uvtest.check_warnings(
+        UserWarning, "ALL FILES FAILED ON READ",
+    ):
+        uvb3.read(filenames, skip_bad_files=True)
+
+
+@pytest.mark.parametrize(
+    "filename", [cst_yaml_file, mwa_beam_file, casa_beamfits],
+)
+def test_from_file(filename):
+    """Test from file produces same the results as reading explicitly."""
+    uvb = UVBeam()
+    # don't run checks because of casa_beamfits, we'll do that later
+    uvb.read(filename, run_check=False)
+    uvb2 = UVBeam.from_file(filename, run_check=False)
+    uvb.read(filename, run_check=False)
+    # hera casa beam is missing some parameters but we just want to check
+    # that reading is going okay
+    if filename == casa_beamfits:
+        # fill in missing parameters
+        for _uvb in [uvb, uvb2]:
+            _uvb.data_normalization = "peak"
+            _uvb.feed_name = "casa_ideal"
+            _uvb.feed_version = "v0"
+            _uvb.model_name = "casa_airy"
+            _uvb.model_version = "v0"
+
+            # this file is actually in an orthoslant projection RA/DEC at zenith at a
+            # particular time.
+            # For now pretend it's in a zenith orthoslant projection
+            _uvb.pixel_coordinate_system = "orthoslant_zenith"
+    # double check the files are valid
+    assert uvb.check()
+    assert uvb2.check()
+    assert uvb == uvb2
